"use strict";
var Rpc_1 = require("./Rpc");
var Connector = (function () {
    function Connector(ws_url) {
        this.ws_url = ws_url;
        this.callbacks = {};
        this.requests_queue = [];
        this.open = false;
    }
    Connector.prototype.addRequest = function (method, params, cb) {
        var request = new Rpc_1.RpcRequest(method, params);
        var callback = function (resp) {
            if (resp.error) {
                cb(undefined, resp.error);
                return;
            }
            cb(resp.result);
        };
        this.callbacks[request.id] = callback;
        this.send(request);
    };
    Connector.prototype.send = function (request) {
        if (request) {
            this.requests_queue.push(request);
        }
        if (this.open) {
            var current = this.requests_queue.shift();
            if (current) {
                this.socket.send(current.serialize());
                this.send();
            }
        }
    };
    Connector.prototype.recieve = function (data) {
        var resp = new Rpc_1.RpcResponse(data);
        this.callbacks[resp.id](resp);
        delete this.callbacks[resp.id];
    };
    Connector.prototype.connect = function () {
        var _this = this;
        var socket = this.socket = new WebSocket(this.ws_url);
        socket.onopen = function () {
            _this.open = true;
            _this.send();
        };
        socket.onclose = function (event) {
            _this.open = false;
            _this.connect();
        };
        socket.onmessage = function (event) {
            _this.recieve(event.data);
        };
        socket.onerror = function (error) {
            throw error;
        };
    };
    return Connector;
}());
exports.__esModule = true;
exports["default"] = Connector;
