declare module 'ws-jsonrpc' {
    /**
     * WebSocket client, that works with JSON-RPC 2.0 protocol
     * @see http://www.jsonrpc.org/specification
     */
    export class Client {
        /**
         *
         * @param url - websocket server url
         * @param connections - number of simultaneous connections
         */
        constructor(url: string, connections?: number);

        /**
         *
         * @param method - RPC method name
         * @param params - object of any type with parameters (may be also string or number)
         * @param cb - callback, that accepts RPC result and RPC error objects
         */
        send<TParams, TResult>(method: string, params: TParams, cb: RpcCallback<TResult>): void;
    }
    export type RpcCallback<TResult> = (result: TResult, err?: RpcError) => void;

    export interface RpcError {
        code: string | number;
        message: string;
        data?: any;
    }
}

