import {RpcCallback} from "./Main";
import {RpcRequest, RpcResponse} from "./Rpc";
export default class Connector {
    constructor(private ws_url: string) {

    }

    addRequest<TParams, TResult>(method: string, params: TParams, cb: RpcCallback<TResult>) {
        let request = new RpcRequest(method, params);

        let callback = (resp: RpcResponse<TResult>) => {
            if (resp.error) {
                cb(undefined, resp.error);
                return;
            }
            cb(resp.result);
        };

        this.callbacks[request.id] = callback;
        this.send(request);
    }

    private send(request?: RpcRequest) {
        if (request) {
            this.requests_queue.push(request);
        }
        if (this.open) {
            let current = this.requests_queue.shift();
            if (current) {
                this.socket.send(current.serialize());
                this.send();
            }
        }
    }

    private recieve(data: string) {
        let resp = new RpcResponse(data);
        this.callbacks[resp.id](resp);
        delete this.callbacks[resp.id];
    }

    private connect() {
        let socket = this.socket = new WebSocket(this.ws_url);
        socket.onopen = ()=>{
            this.open = true;
            this.send();
        };
        socket.onclose = (event)=>{
            this.open = false;
            this.connect();
        };
        socket.onmessage = (event: MessageEvent) => {
            this.recieve(event.data);
        };
        socket.onerror = (error) => {
            throw error;
        };
    }

    private callbacks: {[id: string]: (resp: RpcResponse<any>)=>void} = {};

    private requests_queue: RpcRequest[] = [];

    private socket: WebSocket;

    private open: boolean = false;
}
